import { deepMerge } from './utils';
import { MessageRenderer } from 'bueno/locale';
import { VerbBuilder } from 'bueno/locale';
declare type VerbVariant = {
    pos_: VerbBuilder;
    neg_: VerbBuilder;
};
export declare type Verb<A> = {
    shouldBe_: VerbVariant;
    when_: VerbVariant;
    passive_: VerbVariant;
    irIs_: A[];
    irIsNot_: A[];
};
export declare type Message<Atom> = {
    k: 'msg';
    disj_: {
        [byPath in string]: {
            verbs_: {
                [byVerb in string]: Verb<Atom>;
            };
        };
    };
};
declare type Implication<Atom> = {
    k: 'when';
    cond: IR<Atom>[];
    conseq: IR<Atom>;
};
export declare type IR<Atom> = Message<Atom> | Implication<Atom>;
export declare function pickVerb(group: Verb<any>, path: string, inWhen: boolean, negated: boolean): VerbBuilder;
export declare type RenderingOptions<A, Out> = {
    fromstr: (s: string) => Out;
    or: (words: Out[]) => Out;
    and: (words: Out[]) => Out;
    noun: (noun: string, name: string, type: string) => Out;
    when: (conseq: Out, cond: Out) => Out;
    elem: (path: (string | number)[]) => Out;
    path: (path: string[]) => Out;
    atom: (ir: A[], props: RenderingOptions<A, Out>) => Out;
    words: (o: Out[]) => Out;
    pathAtKey: (path: Out, verb: Out, byKey: boolean) => Out;
    finalize: (o: Out) => Out;
};
export declare type Context = {
    atPath: string | null;
};
export declare function mkRenderer<A, Out>(props: RenderingOptions<A, Out>): MessageRenderer<IR<A>, Out>;
export declare function createVerb<A>(is: VerbBuilder, not: VerbBuilder, when: VerbBuilder, whenNot: VerbBuilder, passive: VerbBuilder, passiveNot: VerbBuilder): Verb<A>;
export declare function setVerb<A>(verb: Verb<A>): (ir: IR<A>) => IR<A>;
export declare function atom<A>(atom: A): IR<A>;
declare function fromString(noun_: string, name_?: string): IR<any>;
declare function buildWhen<A>(conseq: IR<A>, cond: IR<A>[]): IR<A>;
export declare function not<A>(conj: IR<A>): IR<A>;
export declare function at<A, K extends string>(pfx: K, r: IR<A>): IR<A>;
export declare function verb<A>(is: VerbBuilder | string, not: VerbBuilder | string, when?: VerbBuilder | string, whenNot?: VerbBuilder | string, passive?: VerbBuilder | string, passiveNot?: VerbBuilder | string): (msgs: string | IR<A>, name?: string) => IR<A>;
export declare const defaultBuilder: {
    not: typeof not;
    either: typeof deepMerge;
    when: typeof buildWhen;
    at: typeof at;
    fromString: typeof fromString;
    atEvery<A>(rs: IR<A>): IR<A>;
    verb: typeof verb;
};
export {};
